package com.pj.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.pj.api.client.async_server.AsyncServerInterface;
import com.pj.api.client.level_one_server.LevelOneServerInterface;
import com.pj.api.client.transport.TransportInterface;
import com.pj.api.dto.*;
import com.pj.dto.DataDto;
import com.pj.enummj.DataType;
import com.pj.enummj.FinishStatus;
import com.pj.ht_borders_list.HtBordersList;
import com.pj.ht_borders_list.HtBordersListMapper;
import com.pj.ht_byte.HtByte;
import com.pj.ht_byte.HtByteMapper;
import com.pj.ht_import_declare_people.HtImportDeclarePeople;
import com.pj.ht_import_declare_people.HtImportDeclarePeopleMapper;
import com.pj.ht_pass_card_record.HtPassCardRecord;
import com.pj.ht_pass_card_record.HtPassCardRecordMapper;
import com.pj.ht_trade_settlement.HtTradeSettlement;
import com.pj.ht_trade_settlement.HtTradeSettlementMapper;
import com.pj.rabbitmq.RabbitMQ;
import com.pj.utils.ht.AESUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


/**
 * @Author Mechrevo
 * @Date 2023 08 27 09 57
 **/
@Configuration
@EnableScheduling
public class HtByteTask {

    /**
     * 航通字节mapper
     */
    @Autowired
    private HtByteMapper htByteMapper;
    /**
     * 一级市场
     */
    @Autowired
    private LevelOneServerInterface levelOneServerInterface;
    /**
     * mq测试
     */
    @Autowired
    private AsyncServerInterface asyncServerInterface;
    /**
     * 货运
     */
    @Autowired
    private TransportInterface transportInterface;
    /**
     * 交易结算信息
     */
    @Autowired
    private HtImportDeclarePeopleMapper htImportDeclarePeopleMapper;
    /**
     * 放行记录（过卡记录）
     */
    @Autowired
    private HtPassCardRecordMapper htPassCardRecordMapper;
    /**
     * 方法抽取
     */
    @Autowired
    private MethodHtByteTask methodHtByteTask;
    /**
     * 交易结算信息
     */
    @Autowired
    private HtTradeSettlementMapper htTradeSettlementMapper;
    /**
     * 边民购买记录
     */
    @Autowired
    private HtBordersListMapper htBordersListMapper;

    /**
     * 启动定时任务
     *
     *      * 报文类型
     *      * 001=边民备案
     *      * 002=商铺备案
     *      * 003=边民互助组备案
     *      * 004=边民合作社备案
     *      * 005=商品基础信息
     *      * 006=交易结算信息
     *      * 007=放行记录（过卡记录）
     */
    @Scheduled(cron = "*/9 * * * * ?")  // 测试阶段,每9秒扫描一次
    private void htByteTask(){
        /*  测试mq */
//        OrderDto orderDtoById = levelOneServerInterface.getOrderDtoById(8833342005355478419L);
//        asyncServerInterface.sendLevelOneOrderDto(orderDtoById, DataType.DATA_TYPE_ONE.getCode());
        /*  测试mq */
        //扫描表内是否有待处理任务
        List<HtByte> byteList = htByteMapper.selectList(new LambdaQueryWrapper<HtByte>().eq(HtByte::getFinishStatus, FinishStatus.FINISH_STATUS_ZERO.getCode()));
        //判断是否有任务,没有则返回
        if(byteList.size() == 0){
            System.out.println("\n  ---  暂无待处理任务。" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) + "\n");
            return;
        }
        //执行任务处理流程
        byteList.forEach(item -> {
            //对象转换
            DataDto dataDto = JSONObject.parseObject(item.getBody(), DataDto.class);
            //数据解析
            String body = AESUtil.decryptCBC(dataDto.getData(), RabbitMQ.ACC_KEY);
            //数据处理完成状态
            boolean flag = false;
            // 根据类型匹配对象
            switch (dataDto.getDataType()){
                case "001":
                    //对象转换
                    PeopleDto peopleDto = JSONObject.parseObject(body, PeopleDto.class);
                    //根据其内的身份证,有则新增,无则修改???
                    flag = levelOneServerInterface.peopleDto(peopleDto);
                    if(!flag){
                        //将完成状态设置成 2 , 表示处理失败
                        item.setFinishStatus(FinishStatus.FINISH_STATUS_TWO.getCode());
                        item.setMessageId(dataDto.getMsgId());
                        item.setDataType(dataDto.getDataType());
                        //保存
                        htByteMapper.updateById(item);
                        System.out.println("\n边民信息处理失败。 messageId = " + item.getMessageId() + "\n");
                        //todo: 发送失败信息给管理员

                        break;
                    }
                    item.setFinishStatus(FinishStatus.FINISH_STATUS_ONE.getCode());
                    item.setMessageId(dataDto.getMsgId());
                    item.setDataType(dataDto.getDataType());
                    htByteMapper.updateById(item);
                    System.out.println("\n边民信息处理成功。 messageId = " + item.getMessageId() + "\n");
                    break;
                case "002":
                    //商铺备案
                    ShopDto shopDto = JSONObject.parseObject(body, ShopDto.class);
                    //保存
                    flag = levelOneServerInterface.shopDto(shopDto);
                    if(!flag){
                        //将完成状态设置成 2 , 表示处理失败
                        item.setFinishStatus(FinishStatus.FINISH_STATUS_TWO.getCode());
                        item.setMessageId(dataDto.getMsgId());
                        item.setDataType(dataDto.getDataType());
                        //保存
                        htByteMapper.updateById(item);
                        System.out.println("\n店铺信息处理失败。 messageId = " + item.getMessageId() + "\n");
                        //todo: 发送失败信息给管理员

                        break;
                    }
                    item.setFinishStatus(FinishStatus.FINISH_STATUS_ONE.getCode());
                    item.setMessageId(dataDto.getMsgId());
                    item.setDataType(dataDto.getDataType());
                    htByteMapper.updateById(item);
                    System.out.println("\n店铺信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    break;
                case "003":
                    //边民互助组备案
                    GroupDto groupDto = JSONObject.parseObject(body, GroupDto.class);
                    //保存
                    flag = levelOneServerInterface.groupDto(groupDto);
                    if(!flag){
                        //将完成状态设置成 2 , 表示处理失败
                        item.setFinishStatus(FinishStatus.FINISH_STATUS_TWO.getCode());
                        item.setMessageId(dataDto.getMsgId());
                        item.setDataType(dataDto.getDataType());
                        //保存
                        htByteMapper.updateById(item);
                        System.out.println("\n互助组信息处理失败。 messageId = " + item.getMessageId() + "\n");
                        //todo: 发送失败信息给管理员

                        break;
                    }
                    item.setFinishStatus(FinishStatus.FINISH_STATUS_ONE.getCode());
                    item.setMessageId(dataDto.getMsgId());
                    item.setDataType(dataDto.getDataType());
                    htByteMapper.updateById(item);
                    System.out.println("\n互助组信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    break;
                case "004":
                    //边民合作社备案
                    CooperativeDto cooperativeDto = JSONObject.parseObject(body, CooperativeDto.class);
                    //保存
                    flag = levelOneServerInterface.cooperativeDto(cooperativeDto);
                    if(!flag){
                        //将完成状态设置成 2 , 表示处理失败
                        item.setFinishStatus(FinishStatus.FINISH_STATUS_TWO.getCode());
                        item.setMessageId(dataDto.getMsgId());
                        item.setDataType(dataDto.getDataType());
                        //保存
                        htByteMapper.updateById(item);
                        System.out.println("\n边民合作社信息处理失败。 messageId = " + item.getMessageId() + "\n");
                        //todo: 发送失败信息给管理员

                        break;
                    }
                    item.setFinishStatus(FinishStatus.FINISH_STATUS_ONE.getCode());
                    item.setMessageId(dataDto.getMsgId());
                    item.setDataType(dataDto.getDataType());
                    htByteMapper.updateById(item);
                    System.out.println("\n边民合作社信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    break;
                case "005":
                    //商品基础信息
                    GoodsDto goodsDto = JSONObject.parseObject(body, GoodsDto.class);
                    //保存
                    flag = levelOneServerInterface.goodsDto(goodsDto);
                    if(!flag){
                        //将完成状态设置成 2 , 表示处理失败
                        item.setFinishStatus(FinishStatus.FINISH_STATUS_TWO.getCode());
                        item.setMessageId(dataDto.getMsgId());
                        item.setDataType(dataDto.getDataType());
                        //保存
                        htByteMapper.updateById(item);
                        System.out.println("\n海关监管商品信息处理失败。 messageId = " + item.getMessageId() + "\n");
                        //todo: 发送失败信息给管理员

                        break;
                    }
                    item.setFinishStatus(FinishStatus.FINISH_STATUS_ONE.getCode());
                    item.setMessageId(dataDto.getMsgId());
                    item.setDataType(dataDto.getDataType());
                    htByteMapper.updateById(item);
                    System.out.println("\n海关监管商品信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    break;
                case "006":
                    //交易结算信息
                    HtTradeSettlement importDeclarePeople = JSONObject.parseObject(body, HtTradeSettlement.class);
                    importDeclarePeople.setFinishStatus(FinishStatus.FINISH_STATUS_ZERO.getCode()); // 默认待处理
                    //保存
                    int insert = htTradeSettlementMapper.insert(importDeclarePeople);
                    if(insert == 1){
                        List<HtBordersList> bordersLists = JSON.parseArray(importDeclarePeople.getBordersList(), HtBordersList.class);
                        //创建预申报单号
                        String platSeqNo = null;
                        //保存
                        if(bordersLists.size() > 0){
                            for(HtBordersList t: bordersLists){
                                platSeqNo = t.getPlatSeqNo();
                                t.setCreateTime(new Date());
                                t.setFinishStatus(FinishStatus.FINISH_STATUS_ZERO.getCode());
                                htBordersListMapper.insert(t);
                            }
                        /**
                         * 接收订单推送消息，边民进行交易确认，支付申请，刷脸认证，系统发起进行交易
                         */
                            List<HtBordersList> htBordersLists = htBordersListMapper.selectList(new LambdaQueryWrapper<HtBordersList>().eq(HtBordersList::getPlatSeqNo, platSeqNo));
                            methodHtByteTask.sendMessageToPeople(htBordersLists);
                        }
                    }
                    break;
                case "007":
                    //放行记录（过卡记录）
                    HtPassCardRecord htPassCardRecord = JSONObject.parseObject(body, HtPassCardRecord.class);
                    htPassCardRecord.setFinishStatus(FinishStatus.FINISH_STATUS_ZERO.getCode()); // 默认待处理
                    //保存
                    htPassCardRecordMapper.insert(htPassCardRecord);
                    break;
            }
        });

    }

}
